% Creation of a signal made of several piecewise %linear chirps
%
% [signal,time] = chirpprofile(fs,P1,P2,...,Pi,...PN)
% [signal,time] = chirpprofile(fs) for the default signal
%
% fs is sampling frequency
% Pi is the frequency profile of the ith chirp
% signal is the sum of N chirps
% time is the time axis
%
% Syntax 
% P: [t1 t2 ... ti ... tend] in seconds
%    [f1 f2 ... fi ... fend] in Hz

function [signal,time,freqprof] =chirpprofile(fs,varargin)
flag=1;

nb=nargin-1;
if nb==0;
  %profiles{1}=[0 50 110 170 230 290 360 400; 400 400 560 400 400 560 400 400];
   profiles{1}=[0 25 55 85 115 145 180 200;200 200 280 200 200 280 200 200];
  nb=1;
  if flag==1;
    %profiles{2}=[0 70 140 210 280 350 400; 560 400 400 560 400 400 500];
    %profiles{3}=[0 100 100 200 400; 560 730 470 470 730];
    %profiles{4}=[0 20 20 120 370 370 400; 590 610 350 350 610 350 350];
    profiles{2}=[0 35 70 105 140 175 200; 280 200 200 280 200 200 250];
    profiles{3}=[0 50 50 100 200; 280 365 235 235 365];
    profiles{4}=[0 10 10 60 175 175 200; 285 305 175 175 305 175 175];
    nb=4;
  end
% plot theoretical lines:
% figure, line([0 25 55 85 115 145 180 200]*800,[200 200 280 200 200 280 200 200]/800,'Color','k')
% line([0 35 70 105 140 175 200]*800,[280 200 200 280 200 200 250]/800,'Color','k');
% line([0 50 50 100 200]*800,[280 365 235 235 365]/800,'Color','k');
% line([0 10 10 60 175 175 200]*800,[285 305 175 175 305 175 175]/800,'Color','k');
else
  profiles=varargin;
end

tmax=0;
% check of the Ps and search for tmax
for it=1:nb
  P=profiles{it};

  % size check
  if size(P,1)~=2; error(['Matrix P',num2str(it),' has the wrong number of columns.']); end
  % Shannon check
  if sum(P(2,:)>fs); error(['Matrix P',num2str(it),' will create alias. Higher fs is required.']); end
  % time checks
  if size(P,2)==1; error(['Matrix P',num2str(it),' contains only one time index. Two are required']); end
  if sum((P(1,:)<0)); error(['Matrix P',num2str(it),' contains negative times.']); end
  if sum( (P(1,2:end)-P(1,1:end-1)) <0 ); error(['Matrix P',num2str(it),' contains non-increasing time indices. Two are required']); end
  tmax=max(tmax,P(1,end));
end

signal=zeros(1,round(tmax*fs)+1);
time=((1:length(signal))-1)/fs;

for it=1:nb
  P=profiles{it};

  freq=zeros(size(signal));
  for tind=1:(size(P,2)-1);
    indstart=round(P(1,tind)*fs)+1; indend=round(P(1,tind+1)*fs+1);
    freq(indstart:indend)=linspace(P(2,tind),P(2,tind+1),indend-indstart+1);
  end;
  freqprof(it,:)=freq;
  signal=signal+exp(2*pi*i*cumsum(freq/fs));
end
